@@ -2,7 +2,7 @@
 #ifndef _GLIBMM_USTRING_H
 #define _GLIBMM_USTRING_H
 
-/* $Id: ustring.h 481 2007-12-28 12:22:13Z murrayc $ */
+/* $Id: ustring.h 711 2008-08-06 08:34:07Z murrayc $ */
 
 /* Copyright (C) 2002 The gtkmm Development Team
  *
@@ -1228,6 +1228,8 @@ ustring ustring::format(const T1& a1, const T2& a2, const T3& a3, const T4& a4,
   return buf.to_string();
 }
 
+/** An inner class used by ustring.
+ */
 template <class T>
 class ustring::Stringify
 {
@@ -1240,10 +1242,14 @@ private:
 
 public:
   explicit inline Stringify(const T& arg) : string_ (ustring::format(arg)) {}
-  explicit inline Stringify(const char* arg) : string_ (arg) {}
+
+  //TODO: Why is this here? See the template specialization:
+  explicit inline Stringify(const char* arg) : string_(arg) {}
+
   inline const ustring* ptr() const { return &string_; }
 };
 
+/// A template specialization for Stringify<ustring>:
 template <>
 class ustring::Stringify<ustring>
 {
@@ -1255,15 +1261,54 @@ private:
   Stringify<ustring>& operator=(const ustring::Stringify<ustring>&);
 
 public:
-  explicit inline Stringify(const ustring& arg) : string_ (arg) {}
+  explicit inline Stringify(const ustring& arg) : string_(arg) {}
   inline const ustring* ptr() const { return &string_; }
 };
 
+/** A template specialization for Stringify<const char*>,
+ * because the regular template has ambiguous constructor overloads for char*.
+ */ 
+template <>
+class ustring::Stringify<const char*>
+{
+private:
+  const ustring& string_;
+
+  // noncopyable
+  Stringify(const ustring::Stringify<const char*>&);
+  Stringify<ustring>& operator=(const ustring::Stringify<const char*>&);
+
+public:
+  explicit inline Stringify(const char* arg) : string_(arg) {}
+  inline const ustring* ptr() const { return &string_; }
+};
+
+/* TODO: I can't seem to specify a template specialization for Stringify with a string literal. murrayc.
+
+ * A template specialization for Stringify<char[N]> (for string literals),
+ * because the regular template has ambiguous constructor overloads for char*.
+
+template <std::size_t N>
+class ustring::Stringify<const char[N]>
+{
+private:
+  const ustring& string_;
+
+  // noncopyable
+  Stringify(const ustring::Stringify<const char[N]>&);
+  Stringify<ustring>& operator=(const ustring::Stringify<const char[N]>&);
+
+public:
+  explicit inline Stringify(const char arg[N]) : string_(arg) {}
+  inline const ustring* ptr() const { return &string_; }
+};
+*/
+
 template <class T1>
 inline // static
 ustring ustring::compose(const ustring& fmt, const T1& a1)
 {
-  const ustring::Stringify<T1> s1 (a1);
+  const ustring::Stringify<T1> s1(a1);
 
   const ustring *const argv[] = { s1.ptr() };
   return ustring::compose_argv(fmt, G_N_ELEMENTS(argv), argv);
@@ -1273,8 +1318,8 @@ template <class T1, class T2>
 inline // static
 ustring ustring::compose(const ustring& fmt, const T1& a1, const T2& a2)
 {
-  const ustring::Stringify<T1> s1 (a1);
-  const ustring::Stringify<T2> s2 (a2);
+  const ustring::Stringify<T1> s1(a1);
+  const ustring::Stringify<T2> s2(a2);
 
   const ustring *const argv[] = { s1.ptr(), s2.ptr() };
   return ustring::compose_argv(fmt, G_N_ELEMENTS(argv), argv);
@@ -1285,9 +1330,9 @@ inline // static
 ustring ustring::compose(const ustring& fmt,
                          const T1& a1, const T2& a2, const T3& a3)
 {
-  const ustring::Stringify<T1> s1 (a1);
-  const ustring::Stringify<T2> s2 (a2);
-  const ustring::Stringify<T3> s3 (a3);
+  const ustring::Stringify<T1> s1(a1);
+  const ustring::Stringify<T2> s2(a2);
+  const ustring::Stringify<T3> s3(a3);
 
   const ustring *const argv[] = { s1.ptr(), s2.ptr(), s3.ptr() };
   return ustring::compose_argv(fmt, G_N_ELEMENTS(argv), argv);
@@ -1298,10 +1343,10 @@ inline // static
 ustring ustring::compose(const ustring& fmt,
                          const T1& a1, const T2& a2, const T3& a3, const T4& a4)
 {
-  const ustring::Stringify<T1> s1 (a1);
-  const ustring::Stringify<T2> s2 (a2);
-  const ustring::Stringify<T3> s3 (a3);
-  const ustring::Stringify<T4> s4 (a4);
+  const ustring::Stringify<T1> s1(a1);
+  const ustring::Stringify<T2> s2(a2);
+  const ustring::Stringify<T3> s3(a3);
+  const ustring::Stringify<T4> s4(a4);
 
   const ustring *const argv[] = { s1.ptr(), s2.ptr(), s3.ptr(), s4.ptr() };
   return ustring::compose_argv(fmt, G_N_ELEMENTS(argv), argv);
@@ -1313,11 +1358,11 @@ ustring ustring::compose(const ustring& fmt,
                          const T1& a1, const T2& a2, const T3& a3,
                          const T4& a4, const T5& a5)
 {
-  const ustring::Stringify<T1> s1 (a1);
-  const ustring::Stringify<T2> s2 (a2);
-  const ustring::Stringify<T3> s3 (a3);
-  const ustring::Stringify<T4> s4 (a4);
-  const ustring::Stringify<T5> s5 (a5);
+  const ustring::Stringify<T1> s1(a1);
+  const ustring::Stringify<T2> s2(a2);
+  const ustring::Stringify<T3> s3(a3);
+  const ustring::Stringify<T4> s4(a4);
+  const ustring::Stringify<T5> s5(a5);
 
   const ustring *const argv[] = { s1.ptr(), s2.ptr(), s3.ptr(), s4.ptr(), s5.ptr() };
   return ustring::compose_argv(fmt, G_N_ELEMENTS(argv), argv);
@@ -1329,12 +1374,12 @@ ustring ustring::compose(const ustring& fmt,
                          const T1& a1, const T2& a2, const T3& a3,
                          const T4& a4, const T5& a5, const T6& a6)
 {
-  const ustring::Stringify<T1> s1 (a1);
-  const ustring::Stringify<T2> s2 (a2);
-  const ustring::Stringify<T3> s3 (a3);
-  const ustring::Stringify<T4> s4 (a4);
-  const ustring::Stringify<T5> s5 (a5);
-  const ustring::Stringify<T6> s6 (a6);
+  const ustring::Stringify<T1> s1(a1);
+  const ustring::Stringify<T2> s2(a2);
+  const ustring::Stringify<T3> s3(a3);
+  const ustring::Stringify<T4> s4(a4);
+  const ustring::Stringify<T5> s5(a5);
+  const ustring::Stringify<T6> s6(a6);
 
   const ustring *const argv[] = { s1.ptr(), s2.ptr(), s3.ptr(), s4.ptr(),
                                   s5.ptr(), s6.ptr() };
@@ -1347,13 +1392,13 @@ ustring ustring::compose(const ustring& fmt,
                          const T1& a1, const T2& a2, const T3& a3,
                          const T4& a4, const T5& a5, const T6& a6, const T7& a7)
 {
-  const ustring::Stringify<T1> s1 (a1);
-  const ustring::Stringify<T2> s2 (a2);
-  const ustring::Stringify<T3> s3 (a3);
-  const ustring::Stringify<T4> s4 (a4);
-  const ustring::Stringify<T5> s5 (a5);
-  const ustring::Stringify<T6> s6 (a6);
-  const ustring::Stringify<T7> s7 (a7);
+  const ustring::Stringify<T1> s1(a1);
+  const ustring::Stringify<T2> s2(a2);
+  const ustring::Stringify<T3> s3(a3);
+  const ustring::Stringify<T4> s4(a4);
+  const ustring::Stringify<T5> s5(a5);
+  const ustring::Stringify<T6> s6(a6);
+  const ustring::Stringify<T7> s7(a7);
 
   const ustring *const argv[] = { s1.ptr(), s2.ptr(), s3.ptr(), s4.ptr(),
                                   s5.ptr(), s6.ptr(), s7.ptr() };
@@ -1368,14 +1413,14 @@ ustring ustring::compose(const ustring& fmt,
                          const T4& a4, const T5& a5, const T6& a6,
                          const T7& a7, const T8& a8)
 {
-  const ustring::Stringify<T1> s1 (a1);
-  const ustring::Stringify<T2> s2 (a2);
-  const ustring::Stringify<T3> s3 (a3);
-  const ustring::Stringify<T4> s4 (a4);
-  const ustring::Stringify<T5> s5 (a5);
-  const ustring::Stringify<T6> s6 (a6);
-  const ustring::Stringify<T7> s7 (a7);
-  const ustring::Stringify<T8> s8 (a8);
+  const ustring::Stringify<T1> s1(a1);
+  const ustring::Stringify<T2> s2(a2);
+  const ustring::Stringify<T3> s3(a3);
+  const ustring::Stringify<T4> s4(a4);
+  const ustring::Stringify<T5> s5(a5);
+  const ustring::Stringify<T6> s6(a6);
+  const ustring::Stringify<T7> s7(a7);
+  const ustring::Stringify<T8> s8(a8);
 
   const ustring *const argv[] = { s1.ptr(), s2.ptr(), s3.ptr(), s4.ptr(),
                                   s5.ptr(), s6.ptr(), s7.ptr(), s8.ptr() };
@@ -1390,15 +1435,15 @@ ustring ustring::compose(const ustring& fmt,
                          const T4& a4, const T5& a5, const T6& a6,
                          const T7& a7, const T8& a8, const T9& a9)
 {
-  const ustring::Stringify<T1> s1 (a1);
-  const ustring::Stringify<T2> s2 (a2);
-  const ustring::Stringify<T3> s3 (a3);
-  const ustring::Stringify<T4> s4 (a4);
-  const ustring::Stringify<T5> s5 (a5);
-  const ustring::Stringify<T6> s6 (a6);
-  const ustring::Stringify<T7> s7 (a7);
-  const ustring::Stringify<T8> s8 (a8);
-  const ustring::Stringify<T9> s9 (a9);
+  const ustring::Stringify<T1> s1(a1);
+  const ustring::Stringify<T2> s2(a2);
+  const ustring::Stringify<T3> s3(a3);
+  const ustring::Stringify<T4> s4(a4);
+  const ustring::Stringify<T5> s5(a5);
+  const ustring::Stringify<T6> s6(a6);
+  const ustring::Stringify<T7> s7(a7);
+  const ustring::Stringify<T8> s8(a8);
+  const ustring::Stringify<T9> s9(a9);
 
   const ustring *const argv[] = { s1.ptr(), s2.ptr(), s3.ptr(), s4.ptr(),
                                   s5.ptr(), s6.ptr(), s7.ptr(), s8.ptr(), s9.ptr() };
@@ -1500,31 +1545,59 @@ inline bool operator>=(const char* lhs, const ustring& rhs)
 
 /** @relates Glib::ustring */
 inline ustring operator+(const ustring& lhs, const ustring& rhs)
-  { ustring temp (lhs); temp += rhs; return temp; }
+{
+  ustring temp(lhs);
+  temp += rhs;
+  return temp;
+}
 
 /** @relates Glib::ustring */
 inline ustring operator+(const ustring& lhs, const char* rhs)
-  { ustring temp (lhs); temp += rhs; return temp; }
+{
+  ustring temp(lhs);
+  temp += rhs;
+  return temp;
+}
 
 /** @relates Glib::ustring */
 inline ustring operator+(const char* lhs, const ustring& rhs)
-  { ustring temp (lhs); temp += rhs; return temp; }
+{
+  ustring temp(lhs);
+  temp += rhs;
+  return temp;
+}
 
 /** @relates Glib::ustring */
 inline ustring operator+(const ustring& lhs, gunichar rhs)
-  { ustring temp (lhs); temp += rhs; return temp; }
+{
+  ustring temp(lhs);
+  temp += rhs;
+  return temp;
+}
 
 /** @relates Glib::ustring */
 inline ustring operator+(gunichar lhs, const ustring& rhs)
-  { ustring temp (1, lhs); temp += rhs; return temp; }
+{
+  ustring temp(1, lhs);
+  temp += rhs;
+  return temp;
+}
 
 /** @relates Glib::ustring */
 inline ustring operator+(const ustring& lhs, char rhs)
-  { ustring temp (lhs); temp += rhs; return temp; }
+{
+  ustring temp(lhs);
+  temp += rhs;
+  return temp;
+}
 
 /** @relates Glib::ustring */
 inline ustring operator+(char lhs, const ustring& rhs)
-  { ustring temp (1, lhs); temp += rhs; return temp; }
+{
+  ustring temp(1, lhs);
+  temp += rhs;
+  return temp;
+}
 
 } // namespace Glib
 
